package de.fhdw.wtf.tooling.editor;

import org.eclipse.jface.text.IAutoEditStrategy;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.source.DefaultAnnotationHover;
import org.eclipse.jface.text.source.IAnnotationHover;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

public class WTFConfiguration extends SourceViewerConfiguration {
	
	private WTFRuleScanner scanner;
	private WTFCommentScanner commentScanner;
	private final ColorManager colorManager;
	
	public WTFConfiguration(final ColorManager colorManager) {
		this.colorManager = colorManager;
	}
	
	@Override
	public String[] getConfiguredContentTypes(final ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE, WTFPartitionScanner.WTF_COMMENT_PARTITION };
	}
	
	@Override
	public IAutoEditStrategy[] getAutoEditStrategies(final ISourceViewer sourceViewer, final String contentType) {
		final IAutoEditStrategy strategy = new WTFAutoClosingBraceAssist();
		return new IAutoEditStrategy[] { strategy };
	}
	
	@Override
	public String getConfiguredDocumentPartitioning(final ISourceViewer sourceViewer) {
		return WTFPartitionScanner.WTF_PARTITIONER;
	}
	
	protected WTFCommentScanner getCommentScanner() {
		if (this.commentScanner == null) {
			this.commentScanner = new WTFCommentScanner(this.colorManager);
		}
		return this.commentScanner;
	}
	
	protected WTFRuleScanner getScanner() {
		if (this.scanner == null) {
			this.scanner = WTFRuleScanner.create(this.colorManager);
		}
		return this.scanner;
	}
	
	@Override
	public IContentAssistant getContentAssistant(final ISourceViewer sourceViewer) {
		final ContentAssistant assistant = new ContentAssistant();
		final IContentAssistProcessor assisstProcessor = new CompletionProcessor();
		assistant.setContentAssistProcessor(assisstProcessor, IDocument.DEFAULT_CONTENT_TYPE);
		assistant.setInformationControlCreator(this.getInformationControlCreator(sourceViewer));
		return assistant;
	}
	
	@Override
	public IPresentationReconciler getPresentationReconciler(final ISourceViewer sourceViewer) {
		final PresentationReconciler reconciler = new PresentationReconciler();
		
		final DefaultDamagerRepairer dr = new DefaultDamagerRepairer(this.getScanner());
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);
		
		final NonRuleBasedDamagerRepairer ndr =
				new NonRuleBasedDamagerRepairer(new TextAttribute(
						this.colorManager.getColor(WTFColorConstants.WTF_COMMENT)));
		reconciler.setDamager(ndr, WTFPartitionScanner.WTF_COMMENT_PARTITION);
		reconciler.setRepairer(ndr, WTFPartitionScanner.WTF_COMMENT_PARTITION);
		
		return reconciler;
	}
	
	@Override
	public IAnnotationHover getAnnotationHover(final ISourceViewer sourceViewer) {
		return new DefaultAnnotationHover();
	}
}
